/*
 * This is the keyboards LED driver for Microsoft Windows desktop applications
 * (It will not work properly as service .)
 * 
 * In order to switch keyboard LEDs, the keyboard state is switched via
 * virtual keys:
 * 	NUM LOCK		=	VK_NUMLOCK
 * 	CAPS LOCK		=	VK_CAPITAL
 * 	SCROLL LOCK		=	VK_SCROLL
 * 
 * Therefore SendInput and GetAsyncKeyState are used, useful links:
 * http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731%28v=vs.85%29.aspx
 * http://msdn.microsoft.com/en-us/library/windows/desktop/ms646293%28v=vs.85%29.aspx
 * http://msdn.microsoft.com/en-us/library/windows/desktop/ms646310%28v=vs.85%29.aspx
 * 
 * For checking keypresses "kbhit()" and "getch()" are used
 * 
 * Stephan Baerwolf (stephan@matrixstorm.com), Ilmenau 2013
 */

#define __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028 1

#include "keyboard.h"

#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>

#include <windows.h>
#include <winuser.h>
#include <winable.h>
#include <conio.h>

static uint64_t initialized = 0;

static bool initialstate_numlock;
static bool initialstate_capslock;
static bool initialstate_scrolllock;

static int __getflags(int virtualkey, bool *cur_state) {
  (*cur_state)=GetKeyState(virtualkey)!=0;
  return 0;
}

static int __setflags(int virtualkey, bool state) {
    bool localstate;
    INPUT skey;
    memset(&skey, 0, sizeof(skey));
    __getflags(virtualkey, &localstate);
    if (localstate!=state) {
      skey.type=INPUT_KEYBOARD;
      skey.ki.wVk=virtualkey;
      SendInput(1, &skey, sizeof(skey));

      skey.ki.dwFlags=KEYEVENTF_KEYUP;
      SendInput(1, &skey, sizeof(skey));
    }
    return 0;
}


int keyboard_isinitialized(void) {
  return (initialized>0);
}

int keyboard_initialize(void) {
  if (initialized==0) {

    // do sth. here:
    __getflags(VK_NUMLOCK, &initialstate_numlock);
    __getflags(VK_CAPITAL, &initialstate_capslock);
    __getflags(VK_SCROLL, &initialstate_scrolllock);

    initialized=1;
    return 0;
  }
  initialized++;
  return 1;
}

int keyboard_finalize(void) {
  if (initialized > 0) {
    initialized--;
    if (initialized==0) {
      
      // undo sth. here:
      __setflags(VK_NUMLOCK, initialstate_numlock);
      __setflags(VK_CAPITAL, initialstate_capslock);
      __setflags(VK_SCROLL, initialstate_scrolllock);
      
      return 0;
    } else return 1;
  }
  return 2;
}

int keyboard_keypressed(void) {
  return kbhit();
}

size_t keyboard_clearkeybufferEx(void* buffer, size_t buffersize) {
  size_t result = 0;
  char *_mybuf  = (void*)buffer;

  while (keyboard_keypressed()) {
    if (result < buffersize)	_mybuf[result]=getch();
    else			getch();
    result++;
  }

  return result;
}
size_t keyboard_clearkeybuffer(void) {
  return keyboard_clearkeybufferEx(NULL, 0);
}

int keyboard_setled(int leds, int state) {
  if (leds & KEYBOARD_LED_NUMLOCK)	__setflags(VK_NUMLOCK, state & KEYBOARD_LED_NUMLOCK);
  if (leds & KEYBOARD_LED_CAPSLOCK)	__setflags(VK_CAPITAL, state & KEYBOARD_LED_CAPSLOCK);
  if (leds & KEYBOARD_LED_SCROLLLOCK)	__setflags(VK_SCROLL , state & KEYBOARD_LED_SCROLLLOCK);
}

int keyboard_toggleled(int leds) {
  bool num, cap, scr;
  char rcvleds = 0;

  __getflags(VK_NUMLOCK, &num);
  __getflags(VK_CAPITAL, &cap);
  __getflags(VK_SCROLL, &scr);

  if (!num) rcvleds |= KEYBOARD_LED_NUMLOCK;
  if (!cap) rcvleds |= KEYBOARD_LED_CAPSLOCK;
  if (!scr) rcvleds |= KEYBOARD_LED_SCROLLLOCK;
  
  return keyboard_setled(leds, rcvleds);
}
